@@ -18,7 +18,9 @@ N_("	arrows / j,k	scroll list\n"),
 N_("	enter		view/edit item\n"),
 N_("	a		add item\n"),
 N_("	r / del		remove selected items\n"),
+N_("	M		merge selected items (into top one)\n"),
 N_("	D		duplicate item\n"),
+N_("	U		remove duplicates\n"),
 "\n",
 N_("	space		select item\n"),
 N_("	+		select all\n"),
@@ -52,14 +54,17 @@ NULL
 static char *editorhelp[] = {
 
 "\n",
-N_("	a,c,p,o,C/arrows/h,l	change tab\n"),
+N_("	arrows/h,l		change tab\n"),
 "\n",
-N_("	1 - 5			edit fields\n"),
+N_("	q			quit to main screen\n"),
+"\n",
+N_("	1 - 5 A - Z		edit fields\n"),
 "\n",
 N_("	k or <			previous item\n"),
 N_("	j or >			next item\n"),
 "\n",
-N_("	r			roll e-mail addresses\n"),
+N_("	r			roll e-mail addresses up\n"),
+N_("	ESC-r			roll e-mail addresses down\n"),
 "\n",
 N_("	u			undo\n"),
 "\n",
